#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winver.h>

#define VER_FILEVERSION        2,15,0,0
#define VER_FILEVERSION_STR    "2.15.0.0\0"

#ifdef GCC_WINDRES
VS_VERSION_INFO		VERSIONINFO
#else
VS_VERSION_INFO		VERSIONINFO
#endif
  FILEVERSION		VER_FILEVERSION
  PRODUCTVERSION	VER_FILEVERSION
  FILEFLAGSMASK		VS_FFI_FILEFLAGSMASK
#ifdef _DEBUG
  FILEFLAGS		VS_FF_DEBUG
#else
  FILEFLAGS		0
#endif
  FILEOS		VOS_NT_WINDOWS32
  FILETYPE		VFT_DLL
  FILESUBTYPE	VFT2_UNKNOWN
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904E4"
    //language ID = U.S. English, char set = Windows, Multilingual
    BEGIN
      VALUE "FileDescription",	"libzint barcode library\0"
      VALUE "FileVersion",	VER_FILEVERSION_STR
      VALUE "InternalName",	"zint.dll\0"
      VALUE "LegalCopyright",	"Copyright © 2025 Robin Stuart & BogDan Vatra\0"
      VALUE "OriginalFilename",	"zint.dll\0"
      VALUE "ProductName",	"libzint\0"
      VALUE "ProductVersion",	VER_FILEVERSION_STR
      VALUE "License",  "BSD-3-Clause\0"
      VALUE "WWW", "https://www.sourceforge.net/projects/zint\0"
    END
  END
  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x0409, 1250
  END
END
