#define WIN32_LEAN_AND_MEAN

#include <windows.h>

#include <winver.h>


#define VER_FILEVERSION        2,15,0,0
#define VER_FILEVERSION_STR    "2.15.0.0\0"

#ifdef GCC_WINDRES

VS_VERSION_INFO		VERSIONINFO

#else

VS_VERSION_INFO VERSIONINFO

#endif

 FILEVERSION VER_FILEVERSION

 PRODUCTVERSION VER_FILEVERSION

  FILEFLAGSMASK		VS_FFI_FILEFLAGSMASK

#ifdef _DEBUG

  FILEFLAGS		VS_FF_DEBUG

#else

  FILEFLAGS		0

#endif

  FILEOS		VOS_NT_WINDOWS32

  FILETYPE		VFT_APP

  FILESUBTYPE	VFT2_UNKNOWN

BEGIN

    BLOCK "StringFileInfo"

    BEGIN

        BLOCK "000904b0"

        BEGIN

            VALUE "CompanyName", "Robin Stuart & BogDan Vatra"

            VALUE "FileDescription", "qtZint barcode generator"

            VALUE "FileVersion", VER_FILEVERSION_STR

            VALUE "InternalName", "qtZint"

            VALUE "LegalCopyright", "Copyright © 2025 Robin Stuart & BogDan Vatra"

            VALUE "License", "GNU General Public License version 3"

            VALUE "OriginalFilename", "qtZint"

            VALUE "ProductName", "Zint Barcode Studio"

            VALUE "ProductVersion", VER_FILEVERSION_STR

            VALUE "WWW", "http://www.zint.org.uk"

        END

    END

    BLOCK "VarFileInfo"

    BEGIN

        VALUE "Translation", 0x9, 1200

    END

END

100 ICON MOVEABLE PURE LOADONCALL DISCARDABLE "zint-qt.ico"
